@@ -8,19 +8,21 @@
 #include <vector>
 
 namespace py = pybind11;
-
 void init_ODESolvers(py::module &m)
 {
     py::class_<goss::ODESolver, std::shared_ptr<goss::ODESolver>> solver_ODESolver(m, "ODESolver");
 
     solver_ODESolver.def("num_states", &goss::ODESolver::num_states)
             .def("is_adaptive", &goss::ODESolver::is_adaptive)
-            .def_readwrite("ldt", &goss::ODESolver::ldt)
             .def("reset", &goss::ODESolver::reset)
             .def("get_internal_time_step", &goss::ODESolver::get_internal_time_step)
+            .def("set_internal_time_step",
+                 [](goss::ODESolver &self, double time_step) {
+                     self.set_internal_time_step(time_step);
+                 })
             .def("solve",
                  [](goss::ODESolver &self, py::array_t<double> y, py::array_t<double> y0,
-                    py::array_t<double> t, const unsigned long num_timesteps, const int skip_n) {
+                    py::array_t<double> t, const unsigned long num_timesteps) {
                      py::buffer_info y_info = y.request();
                      auto y_ptr = static_cast<double *>(y_info.ptr);
 
@@ -30,7 +32,7 @@ void init_ODESolvers(py::module &m)
                      py::buffer_info t_info = t.request();
                      auto t_ptr = static_cast<double *>(t_info.ptr);
 
-                     self.solve(y_ptr, y0_ptr, t_ptr, num_timesteps, skip_n);
+                     self.solve(y_ptr, y0_ptr, t_ptr, num_timesteps);
                  })
             .def("get_ode",
                  [](goss::ODESolver &self) {
@@ -39,6 +41,8 @@ void init_ODESolvers(py::module &m)
             .def("attach",
                  [](goss::ODESolver &self, std::shared_ptr<goss::ODE> ode) { self.attach(ode); });
 
+    // Explicit Solvers
+
     py::class_<goss::ExplicitEuler, goss::ODESolver, std::shared_ptr<goss::ExplicitEuler>>
             solver_ExplicitEuler(m, "ExplicitEuler");
     solver_ExplicitEuler.def(py::init<>())
@@ -64,6 +68,9 @@ void init_ODESolvers(py::module &m)
                      self.forward(y_ptr, t, interval);
                  });
 
+    py::class_<goss::RL2, goss::RL1, std::shared_ptr<goss::RL2>> solver_RL2(m, "RL2");
+    solver_RL2.def(py::init<>()).def(py::init<std::shared_ptr<goss::ODE>>());
+
     py::class_<goss::GRL1, goss::ODESolver, std::shared_ptr<goss::GRL1>> solver_GRL1(m, "GRL1");
     solver_GRL1.def(py::init<>())
             .def(py::init<std::shared_ptr<goss::ODE>>())
@@ -77,6 +84,70 @@ void init_ODESolvers(py::module &m)
                      self.forward(y_ptr, t, interval);
                  });
 
+    py::class_<goss::GRL2, goss::GRL1, std::shared_ptr<goss::GRL2>> solver_GRL2(m, "GRL2");
+    solver_GRL2.def(py::init<>()).def(py::init<std::shared_ptr<goss::ODE>>());
+
+    py::class_<goss::RK2, goss::ODESolver, std::shared_ptr<goss::RK2>> solver_RK2(m, "RK2");
+    solver_RK2.def(py::init<>())
+            .def(py::init<std::shared_ptr<goss::ODE>>())
+            .def("copy", &goss::RK2::copy)
+            .def("forward",
+                 [](goss::RK2 &self, const py::array_t<double> y, double t, double interval) {
+                     py::buffer_info y_info = y.request();
+                     auto y_ptr = static_cast<double *>(y_info.ptr);
+
+                     self.forward(y_ptr, t, interval);
+                 });
+
+    py::class_<goss::RK4, goss::ODESolver, std::shared_ptr<goss::RK4>> solver_RK4(m, "RK4");
+    solver_RK4.def(py::init<>())
+            .def(py::init<std::shared_ptr<goss::ODE>>())
+            .def("copy", &goss::RK4::copy)
+            .def("forward",
+                 [](goss::RK4 &self, const py::array_t<double> y, double t, double interval) {
+                     py::buffer_info y_info = y.request();
+                     auto y_ptr = static_cast<double *>(y_info.ptr);
+
+                     self.forward(y_ptr, t, interval);
+                 });
+
+    // Adaptive Explicit Solvers
+
+    py::class_<goss::AdaptiveExplicitSolver, goss::ODESolver,
+               std::shared_ptr<goss::AdaptiveExplicitSolver>>
+            solver_AdaptiveExplicitSolver(m, "AdaptiveExplicitSolver");
+    solver_AdaptiveExplicitSolver.def("get_atol", &goss::AdaptiveExplicitSolver::get_atol)
+            .def("get_rtol", &goss::AdaptiveExplicitSolver::get_rtol)
+            .def("get_iord", &goss::AdaptiveExplicitSolver::get_iord)
+            .def("get_current_time", &goss::AdaptiveExplicitSolver::get_current_time)
+            .def("get_current_time_step", &goss::AdaptiveExplicitSolver::get_current_time_step)
+            .def("get_num_accepted", &goss::AdaptiveExplicitSolver::get_num_accepted)
+            .def("get_num_rejected", &goss::AdaptiveExplicitSolver::get_num_rejected)
+            .def("set_single_step_mode", [](goss::AdaptiveExplicitSolver &self,
+                                            bool mode) { self.set_single_step_mode(mode); })
+            .def("set_tol", [](goss::AdaptiveExplicitSolver &self, double atol,
+                               double rtol = 1.0e-8) { self.set_tol(atol, rtol); })
+            .def("set_iord",
+                 [](goss::AdaptiveExplicitSolver &self, int iord) { self.set_iord(iord); });
+
+
+    py::class_<goss::RKF32, goss::AdaptiveExplicitSolver, std::shared_ptr<goss::RKF32>>
+            solver_RKF32(m, "RKF32");
+    solver_RKF32.def(py::init<>())
+            .def(py::init<std::shared_ptr<goss::ODE>>())
+            .def("copy", &goss::RKF32::copy)
+            .def_readonly("nfevals", &goss::RKF32::nfevals)
+            .def_readonly("ndtsa", &goss::RKF32::ndtsa)
+            .def_readonly("ndtsr", &goss::RKF32::ndtsr)
+            .def("forward",
+                 [](goss::RKF32 &self, const py::array_t<double> y, double t, double interval) {
+                     py::buffer_info y_info = y.request();
+                     auto y_ptr = static_cast<double *>(y_info.ptr);
+
+                     self.forward(y_ptr, t, interval);
+                 });
+
+
     // Implicit solvers
 
     py::class_<goss::ImplicitODESolver, goss::ODESolver, std::shared_ptr<goss::ImplicitODESolver>>
@@ -100,6 +171,37 @@ void init_ODESolvers(py::module &m)
                      self.compute_factorized_jacobian(y_ptr, t, dt, alpha);
                  });
 
+    py::class_<goss::BasicImplicitEuler, goss::ImplicitODESolver,
+               std::shared_ptr<goss::BasicImplicitEuler>>
+            solver_BasicImplicitEuler(m, "BasicImplicitEuler");
+    solver_BasicImplicitEuler.def(py::init<>())
+            .def(py::init<std::shared_ptr<goss::ODE>>())
+            .def("copy", &goss::BasicImplicitEuler::copy)
+            .def("forward", [](goss::BasicImplicitEuler &self, const py::array_t<double> y,
+                               double t, double interval) {
+                py::buffer_info y_info = y.request();
+                auto y_ptr = static_cast<double *>(y_info.ptr);
+
+                self.forward(y_ptr, t, interval);
+            });
+
+    py::class_<goss::ImplicitEuler, goss::ImplicitODESolver, std::shared_ptr<goss::ImplicitEuler>>
+            solver_ImplicitEuler(m, "ImplicitEuler");
+    solver_ImplicitEuler.def(py::init<>())
+            .def(py::init<std::shared_ptr<goss::ODE>>())
+            .def("copy", &goss::ImplicitEuler::copy)
+            .def_readwrite(
+                    "num_refinements_without_always_recomputing_jacobian",
+                    &goss::ImplicitEuler::num_refinements_without_always_recomputing_jacobian)
+            .def_readwrite("min_dt", &goss::ImplicitEuler::min_dt)
+            .def("forward", [](goss::ImplicitEuler &self, const py::array_t<double> y, double t,
+                               double interval) {
+                py::buffer_info y_info = y.request();
+                auto y_ptr = static_cast<double *>(y_info.ptr);
+
+                self.forward(y_ptr, t, interval);
+            });
+
     py::class_<goss::ThetaSolver, goss::ImplicitODESolver, std::shared_ptr<goss::ThetaSolver>>
             solver_ThetaSolver(m, "ThetaSolver");
     solver_ThetaSolver.def(py::init<>())
@@ -161,6 +263,22 @@ void init_ODESolvers(py::module &m)
 
                      self.compute_ode_jacobian(y_ptr, t);
                  });
+
+    py::class_<goss::ESDIRK4O32, goss::AdaptiveImplicitSolver, std::shared_ptr<goss::ESDIRK4O32>>
+            solver_ESDIRK4O32(m, "ESDIRK4O32");
+    solver_ESDIRK4O32.def(py::init<>())
+            .def(py::init<std::shared_ptr<goss::ODE>>())
+            .def("copy", &goss::ESDIRK4O32::copy)
+            .def_readonly("nfevals", &goss::ESDIRK4O32::nfevals)
+            .def_readonly("ndtsa", &goss::ESDIRK4O32::ndtsa)
+            .def_readonly("ndtsr", &goss::ESDIRK4O32::ndtsr)
+            .def("forward", [](goss::ESDIRK4O32 &self, const py::array_t<double> y, double t,
+                               double interval) {
+                py::buffer_info y_info = y.request();
+                auto y_ptr = static_cast<double *>(y_info.ptr);
+
+                self.forward(y_ptr, t, interval);
+            });
 }
 
 void init_ODE(py::module &m)
@@ -347,6 +465,25 @@ void init_ODE(py::module &m)
 
                      return self.eval_monitored(states_ptr, t, monitored_ptr);
                  })
+            .def("monitored_values",
+                 [](goss::ParameterizedODE &self, const py::array_t<double> states,
+                    const py::array_t<double> t, py::array_t<double> monitored,
+                    py::array_t<double> m) {
+                     py::buffer_info states_info = states.request();
+                     auto states_ptr = static_cast<double *>(states_info.ptr);
+
+                     py::buffer_info t_info = t.request();
+                     auto t_ptr = static_cast<double *>(t_info.ptr);
+
+                     py::buffer_info monitored_info = monitored.request();
+                     auto monitored_ptr = static_cast<double *>(monitored_info.ptr);
+
+                     py::buffer_info m_info = m.request();
+                     auto m_ptr = static_cast<double *>(m_info.ptr);
+
+                     return self.monitored_values(states_ptr, t_ptr, monitored_ptr, m_ptr,
+                                                  t_info.shape[0]);
+                 })
             .def("set_field_parameters",
                  [](goss::ParameterizedODE &self, const py::array_t<double> field_params) {
                      py::buffer_info field_params_info = field_params.request();
